/**
 * @fileOverview Mcuクラスを定義します。
 */
(function(){
var NS=mbedJS;
var MI=MiMicJS;

/**
 * mbedJSを搭載したmbedに接続したインスタンスを生成します。
 * Mcuオブジェクトは、他のペリフェラルクラスをインスタンス化するときに必要です。
 * @name mbedJS.Mcu
 * @constructor
 * @param {string} i_url
 * 接続先のMiMicRPCサービスのアドレスを指定します。
 * @param {HashMap|Generator} i_handler
 * 非同期イベントハンドラの連想配列、又はGeneratorです。
 * <p>
 * 非同期イベントハンドラの場合、関数はイベントハンドラで結果を通知します。
 * <ul>
 * <li>{function()} onNew -
 * インスタンスが使用可能になった時に呼び出されます。
 * </li>
 * <li>{function()} onClose -
 * Close関数のコールバック関数です。
 * </li>
 * <li>{function()} onError -
 * 何らかの異常で通信を継続できないエラーが発生し、コネクションを維持できない時に発生します。
 * このイベントが発生するとコネクションは閉じられれます。
 * </li>
 * </ul>
 * <p>
 * Generatorを指定した場合、コールバック関数の戻り値はyiledの戻り値として取得できます。
 * <p>
 * @return {mbedJS.Mcu}
 * @example //Callback
 * var mcu=new mbedJS.Mcu("192.168.128.39",
 * {
 *   onNew:function(){
 *   },
 *   onClose:function(){
 *   },
 *   onError:function(){
 *   }
 * });
 * @example //Generator
 * var g=function*(){
 * try{
 *   var mcu=new mbedJS.Mcu("192.168.128.39",g);
 *   yield mcu.waitForNew();
 *   yield mcu.close();
 * }catch(e){
 *   mcu.shutdown();
 * }
 * }();
 * g.next();
 */
var CLASS=function Mcu(i_url,i_handler)
{
	var _t=this;
	_t._lc=CLASS;
	_t._has_error=false;
	if(MI.isGenerator(i_handler)){_t._gen=i_handler;}
	else if(i_handler){_t._event=i_handler}
	
	_t._rpc=new MI.Rpc({
		onOpen:function _Mcu_onOpen(){
			if(_t._event.onNew){_t._event.onNew();}
			if(_t._gen){_t._gen.next(_t);}
			_t.lc=null;
		},
		onClose:function _Mcu_onClose(){
			if(_t._lc==CLASS.close){
				if(_t._event.onClose){_t._event.onClose();}
			}else{
				if(_t._event.onError){_t._event.onError();}
			}
			if(_t._gen){
				_t._gen.next(_t);
			}			
			_t.lc=null;
		},
		onError:function _Mcu_onError()
		{
			_t._has_error=true;
			if(_t._event.onError){_t._event.onError();}
			if(_t._gen && _t._lc){
				_t._gen.throw(new MI.MiMicException());
			}
			//@todo MCUにぶら下がってる全てのyieldに対してもExceptionの発生要請？
		}
	});
	//MCUへ接続
	this._rpc.open('ws://'+i_url+'/rpc/');
}
CLASS.prototype=
{
	_lc:null,
	_rpc:null,
	_gen:null,
	_event:{},
	_has_error:false,
	/**
	 * エラー状態であるかを返します。
	 * Generatorモードの場合に、定期実行してインスタンスの状態をチェックできます。
	 * falseの場合、下位のオブジェクトでyieldロックが発生している場合があります。
	 * @name mbedJS.Mcu#hasError
	 * @function
	 * @return {boolean}
	 * true - インスタンスはエラー状態で停止中です。使用できません。
	 * false - インスタンスは動作中です。使用可能です。
	 */
	hasError:function(){
		return _t._has_error;
	},
	/**
	 * Generatorモードのときに使用する関数です。
	 * Generatorモードの時は、yieldと併用してnew Mcu()の完了を待ちます。
	 * @name mbedJS.Mcu#waitForNew
	 * @function
	 */
	waitForNew:function MCU_waitForNew(){
		if(this._lc!=CLASS){throw new MI.MiMicException(MI.Error.NG_ILLEGAL_CALL);}
		this._lc=CLASS.waitForNew;
	},
	/**
	 * RPCを切断します。関数の完了時にonCloseイベントが発生します。
	 * Generatorモードの時は、yieldと併用して完了を待機できます。
	 * @name mbedJS.Mcu#close
	 * @function
	 */
	close:function MCU_close(){
		if(this._gen && this._lc){throw new MI.MiMicException(MI.Error.NG_YIELD_NOT_COMPLETED);}
		this._lc=CLASS.close;
		this._rpc.close();
	},
	/**
	 * コールバック関数を全てキャンセルして、Mcuとの接続をシャットダウンします。
	 * @name mbedJS.Mcu#shutdown
	 * @function
	 */
	shutdown:function MCU_shutdown(){
		this._rpc.shutdown();
	},
	/**
	 * RPCメソッドを実行します。
	 * @name mbedJS.Mcu#rpc
	 * @function
	 * @param {string} m
	 * メソッド名です。
	 * @param {string} p
	 * パラメータ部の文字列です。JSONオブジェクトの配列を記述します。
	 * 配列の要素はプリミティブ型である必要があります。
	 * @param {function(json)} c
	 * RPCが完了したときに呼び出すコールバック関数です。
	 * <ul>
	 * <li>json - 戻り値をJSON文字列としてパースしたObjectです。</li>
	 * </ul>
	 * @return {int}
	 * メソッドのid値を返します。
	 */
	rpc:function Mcu_rpc(m,p,c){
		if(this._has_error){
			throw new MI.MiMicException();
		}
		return this._rpc.sendMethod(m,p,c);
	},
	/** @private */
	addItem:function(o){
		this._items.push(o);
	}
}
NS.Mcu=CLASS;
}());